@@ -3,8 +3,9 @@ import theano.tensor as tt
 
 from pymc3.util import get_variable_name
 from ..math import logsumexp
-from .dist_math import bound
-from .distribution import Discrete, Distribution, draw_values, generate_samples
+from .dist_math import bound, random_choice
+from .distribution import (Discrete, Distribution, draw_values,
+                           generate_samples, _DrawValuesContext)
 from .continuous import get_tau_sd, Normal
 
 
@@ -49,7 +50,7 @@ class Mixture(Distribution):
             lam = pm.Exponential('lam', lam=1, shape=(2,))  # `shape=(2,)` indicates two mixtures.
 
             # As we just need the logp, rather than add a RV to the model, we need to call .dist()
-            components = pm.Poisson.dist(mu=lam, shape=(2,))  
+            components = pm.Poisson.dist(mu=lam, shape=(2,))
 
             w = pm.Dirichlet('w', a=np.array([1, 1]))  # two mixture component weights.
 
@@ -67,6 +68,7 @@ class Mixture(Distribution):
 
             like = pm.Mixture('like', w=w, comp_dists = [pois1, pois2], observed=data)
     """
+
     def __init__(self, w, comp_dists, *args, **kwargs):
         shape = kwargs.pop('shape', ())
 
@@ -95,7 +97,7 @@ class Mixture(Distribution):
 
             if 'mode' not in defaults:
                 defaults.append('mode')
-        except AttributeError:
+        except (AttributeError, ValueError, IndexError):
             pass
 
         super(Mixture, self).__init__(shape, dtype, defaults=defaults,
@@ -109,28 +111,31 @@ class Mixture(Distribution):
 
             return comp_dists.logp(value_)
         except AttributeError:
-            return tt.stack([comp_dist.logp(value) for comp_dist in comp_dists],
-                            axis=1)
+            return tt.squeeze(tt.stack([comp_dist.logp(value)
+                                        for comp_dist in comp_dists],
+                                       axis=1))
 
     def _comp_means(self):
         try:
             return tt.as_tensor_variable(self.comp_dists.mean)
         except AttributeError:
-            return tt.stack([comp_dist.mean for comp_dist in self.comp_dists],
-                            axis=1)
+            return tt.squeeze(tt.stack([comp_dist.mean
+                                        for comp_dist in self.comp_dists],
+                                       axis=1))
 
     def _comp_modes(self):
         try:
             return tt.as_tensor_variable(self.comp_dists.mode)
         except AttributeError:
-            return tt.stack([comp_dist.mode for comp_dist in self.comp_dists],
-                            axis=1)
+            return tt.squeeze(tt.stack([comp_dist.mode
+                                        for comp_dist in self.comp_dists],
+                                       axis=1))
 
-    def _comp_samples(self, point=None, size=None, repeat=None):
+    def _comp_samples(self, point=None, size=None):
         try:
-            samples = self.comp_dists.random(point=point, size=size, repeat=repeat)
+            samples = self.comp_dists.random(point=point, size=size)
         except AttributeError:
-            samples = np.column_stack([comp_dist.random(point=point, size=size, repeat=repeat)
+            samples = np.column_stack([comp_dist.random(point=point, size=size)
                                        for comp_dist in self.comp_dists])
 
         return np.squeeze(samples)
@@ -138,34 +143,48 @@ class Mixture(Distribution):
     def logp(self, value):
         w = self.w
 
-        return bound(logsumexp(tt.log(w) + self._comp_logp(value), axis=-1).sum(),
+        return bound(logsumexp(tt.log(w) + self._comp_logp(value), axis=-1),
                      w >= 0, w <= 1, tt.allclose(w.sum(axis=-1), 1),
                      broadcast_conditions=False)
 
-    def random(self, point=None, size=None, repeat=None):
-        def random_choice(*args, **kwargs):
-            w = kwargs.pop('w')
-            w /= w.sum(axis=-1, keepdims=True)
-            k = w.shape[-1]
+    def random(self, point=None, size=None):
+        with _DrawValuesContext() as draw_context:
+            w = draw_values([self.w], point=point)[0]
+            comp_tmp = self._comp_samples(point=point, size=None)
+        if np.asarray(self.shape).size == 0:
+            distshape = np.asarray(np.broadcast(w, comp_tmp).shape)[..., :-1]
+        else:
+            distshape = np.asarray(self.shape)
 
-            if w.ndim > 1:
-                return np.row_stack([np.random.choice(k, p=w_) for w_ in w])
-            else:
-                return np.random.choice(k, p=w, *args, **kwargs)
-
-        w = draw_values([self.w], point=point)[0]
+        # Normalize inputs
+        w /= w.sum(axis=-1, keepdims=True)
 
         w_samples = generate_samples(random_choice,
-                                     w=w,
+                                     p=w,
                                      broadcast_shape=w.shape[:-1] or (1,),
-                                     dist_shape=self.shape,
+                                     dist_shape=distshape,
                                      size=size).squeeze()
-        comp_samples = self._comp_samples(point=point, size=size, repeat=repeat)
-
-        if comp_samples.ndim > 1:
-            return np.squeeze(comp_samples[np.arange(w_samples.size), w_samples])
+        if (size is None) or (distshape.size == 0):
+            with draw_context:
+                comp_samples = self._comp_samples(point=point, size=size)
+            if comp_samples.ndim > 1:
+                samples = np.squeeze(comp_samples[np.arange(w_samples.size), ..., w_samples])
+            else:
+                samples = np.squeeze(comp_samples[w_samples])
         else:
-            return np.squeeze(comp_samples[w_samples])
+            if w_samples.ndim == 1:
+                w_samples = np.reshape(np.tile(w_samples, size), (size,) + w_samples.shape)
+            samples = np.zeros((size,)+tuple(distshape))
+            with draw_context:
+                for i in range(size):
+                    w_tmp = w_samples[i, :]
+                    comp_tmp = self._comp_samples(point=point, size=None)
+                    if comp_tmp.ndim > 1:
+                        samples[i, :] = np.squeeze(comp_tmp[np.arange(w_tmp.size), ..., w_tmp])
+                    else:
+                        samples[i, :] = np.squeeze(comp_tmp[w_tmp])
+
+        return samples
 
 
 class NormalMixture(Mixture):
@@ -193,15 +212,22 @@ class NormalMixture(Mixture):
         the component standard deviations
     tau : array of floats
         the component precisions
+    comp_shape : shape of the Normal component
+        notice that it should be different than the shape
+        of the mixture distribution, with one axis being
+        the number of components.
 
     Note: You only have to pass in sd or tau, but not both.
     """
-    def __init__(self, w, mu, *args, **kwargs):
+
+    def __init__(self, w, mu, comp_shape=(), *args, **kwargs):
         _, sd = get_tau_sd(tau=kwargs.pop('tau', None),
                            sd=kwargs.pop('sd', None))
+
         self.mu = mu = tt.as_tensor_variable(mu)
         self.sd = sd = tt.as_tensor_variable(sd)
-        super(NormalMixture, self).__init__(w, Normal.dist(mu, sd=sd),
+
+        super(NormalMixture, self).__init__(w, Normal.dist(mu, sd=sd, shape=comp_shape),
                                             *args, **kwargs)
 
     def _repr_latex_(self, name=None, dist=None):
