@@ -6,7 +6,6 @@ import warnings
 
 from six import integer_types
 from joblib import Parallel, delayed
-from tempfile import mkdtemp
 import numpy as np
 import theano.gradient as tg
 
@@ -27,7 +26,9 @@ from tqdm import tqdm
 import sys
 sys.setrecursionlimit(10000)
 
-__all__ = ['sample', 'iter_sample', 'sample_ppc', 'sample_ppc_w', 'init_nuts', 'sample_prior_predictive']
+__all__ = ['sample', 'iter_sample', 'sample_posterior_predictive',
+           'sample_posterior_predictive_w', 'init_nuts',
+           'sample_prior_predictive', 'sample_ppc', 'sample_ppc_w']
 
 STEP_METHODS = (NUTS, HamiltonianMC, Metropolis, BinaryMetropolis,
                 BinaryGibbsMetropolis, Slice, CategoricalGibbsMetropolis)
@@ -243,7 +244,8 @@ def sample(draws=500, step=None, init='auto', n_init=200000, start=None, trace=N
     chains : int
         The number of chains to sample. Running independent chains is important for some
         convergence statistics and can also reveal multiple modes in the posterior. If `None`,
-        then set to either `chains` or 2, whichever is larger. For SMC the default value is 100.
+        then set to either `cores` or 2, whichever is larger. For SMC the number of chains is the
+        number of draws.
     cores : int
         The number of chains to run in parallel. If `None`, set to the number of CPUs in the
         system, but at most 4 (for 'SMC' defaults to 1). Keep in mind that some chains might
@@ -321,23 +323,11 @@ def sample(draws=500, step=None, init='auto', n_init=200000, start=None, trace=N
     if isinstance(step, pm.step_methods.smc.SMC):
         if step_kwargs is None:
             step_kwargs = {}
-        if chains is None:
-            chains = 100
-        if cores is None:
-            cores = 1
-        test_folder = mkdtemp(prefix='SMC_TEST')
-        trace = smc.sample_smc(samples=draws,
-                               chains=chains,
+        trace = smc.sample_smc(draws=draws,
                                step=step,
-                               start=start,
-                               homepath=step_kwargs.get('homepath', test_folder),
-                               stage=step_kwargs.get('stage', 0),
-                               cores=cores,
                                progressbar=progressbar,
                                model=model,
-                               random_seed=random_seed,
-                               rm_flag=step_kwargs.get('rm_flag', True),
-                               **kwargs)
+                               random_seed=random_seed)
     else:
         if cores is None:
             cores = min(4, _cpu_count())
@@ -438,7 +428,7 @@ def sample(draws=500, step=None, init='auto', n_init=200000, start=None, trace=N
 
         sample_args.update(kwargs)
 
-        has_population_samplers = np.any([ isinstance(m, arraystep.PopulationArrayStepShared)
+        has_population_samplers = np.any([isinstance(m, arraystep.PopulationArrayStepShared)
             for m in (step.methods if isinstance(step, CompoundStep) else [step])])
 
         parallel = cores > 1 and chains > 1 and not has_population_samplers
@@ -995,17 +985,28 @@ def _mp_sample(draws, tune, step, chains, cores, chain, random_seed,
             draws, tune, chains, cores, random_seed, start, step,
             chain, progressbar)
         try:
-            with sampler:
-                for draw in sampler:
-                    trace = traces[draw.chain - chain]
-                    if trace.supports_sampler_stats and draw.stats is not None:
-                        trace.record(draw.point, draw.stats)
-                    else:
-                        trace.record(draw.point)
-                    if draw.is_last:
-                        trace.close()
-                        if draw.warnings is not None:
-                            trace._add_warnings(draw.warnings)
+            try:
+                with sampler:
+                    for draw in sampler:
+                        trace = traces[draw.chain - chain]
+                        if (trace.supports_sampler_stats
+                                and draw.stats is not None):
+                            trace.record(draw.point, draw.stats)
+                        else:
+                            trace.record(draw.point)
+                        if draw.is_last:
+                            trace.close()
+                            if draw.warnings is not None:
+                                trace._add_warnings(draw.warnings)
+            except ps.ParallelSamplingError as error:
+                trace = traces[error._chain - chain]
+                trace._add_warnings(error._warnings)
+                for trace in traces:
+                    trace.close()
+
+                multitrace = MultiTrace(traces)
+                multitrace._report._log_summary()
+                raise
             return MultiTrace(traces)
         except KeyboardInterrupt:
             traces, length = _choose_chains(traces, tune)
@@ -1068,8 +1069,8 @@ def stop_tuning(step):
     return step
 
 
-def sample_ppc(trace, samples=None, model=None, vars=None, size=None,
-               random_seed=None, progressbar=True):
+def sample_posterior_predictive(trace, samples=None, model=None, vars=None, size=None,
+                                random_seed=None, progressbar=True):
     """Generate posterior predictive samples from a model given a trace.
 
     Parameters
@@ -1107,7 +1108,7 @@ def sample_ppc(trace, samples=None, model=None, vars=None, size=None,
         nchain = 1
 
     if samples is None:
-        samples = len(trace)
+        samples = sum(len(v) for v in trace._straces.values())
 
     model = modelcontext(model)
 
@@ -1117,31 +1118,23 @@ def sample_ppc(trace, samples=None, model=None, vars=None, size=None,
     if random_seed is not None:
         np.random.seed(random_seed)
 
-    indices = np.random.randint(0, nchain * len_trace, samples)
+    indices = np.arange(samples)
 
     if progressbar:
         indices = tqdm(indices, total=samples)
 
-    varnames = [var.name for var in vars]
-
-    # draw once to inspect the shape
-    var_values = list(zip(varnames,
-                          draw_values(vars, point=model.test_point, size=size)))
     ppc_trace = defaultdict(list)
-    for varname, value in var_values:
-        ppc_trace[varname] = np.zeros((samples,) + value.shape, value.dtype)
-
     try:
-        for slc, idx in enumerate(indices):
+        for idx in indices:
             if nchain > 1:
                 chain_idx, point_idx = np.divmod(idx, len_trace)
-                param = trace._straces[chain_idx].point(point_idx)
+                param = trace._straces[chain_idx % nchain].point(point_idx)
             else:
-                param = trace[idx]
+                param = trace[idx % len_trace]
 
             values = draw_values(vars, point=param, size=size)
             for k, v in zip(vars, values):
-                ppc_trace[k.name][slc] = v
+                ppc_trace[k.name].append(v)
 
     except KeyboardInterrupt:
         pass
@@ -1150,11 +1143,18 @@ def sample_ppc(trace, samples=None, model=None, vars=None, size=None,
         if progressbar:
             indices.close()
 
-    return ppc_trace
+    return {k: np.asarray(v) for k, v in ppc_trace.items()}
 
 
-def sample_ppc_w(traces, samples=None, models=None, weights=None,
-                 random_seed=None, progressbar=True):
+def sample_ppc(*args, **kwargs):
+    """This method is deprecated.  Please use :func:`~sampling.sample_posterior_predictive`"""
+    message = 'sample_ppc() is deprecated.  Please use sample_posterior_predictive()'
+    warnings.warn(message, DeprecationWarning, stacklevel=2)
+    return sample_posterior_predictive(*args, **kwargs)
+
+
+def sample_posterior_predictive_w(traces, samples=None, models=None, weights=None,
+                                    random_seed=None, progressbar=True):
     """Generate weighted posterior predictive samples from a list of models and
     a list of traces according to a set of weights.
 
@@ -1268,7 +1268,7 @@ def sample_ppc_w(traces, samples=None, models=None, weights=None,
         for idx in indices:
             param = trace[idx]
             var = variables[idx]
-            # TODO sample_ppc_w is currently only work for model with
+            # TODO sample_posterior_predictive_w is currently only work for model with
             # one observed.
             ppc[var.name].append(draw_values([var],
                                              point=param,
@@ -1285,6 +1285,13 @@ def sample_ppc_w(traces, samples=None, models=None, weights=None,
     return {k: np.asarray(v) for k, v in ppc.items()}
 
 
+def sample_ppc_w(*args, **kwargs):
+    """This method is deprecated.  Please use :func:`~sampling.sample_posterior_predictive_w`"""
+    message = 'sample_ppc() is deprecated.  Please use sample_posterior_predictive_w()'
+    warnings.warn(message, DeprecationWarning, stacklevel=2)
+    return sample_posterior_predictive_w(*args, **kwargs)
+
+
 def sample_prior_predictive(samples=500, model=None, vars=None, random_seed=None):
     """Generate samples from the prior predictive distribution.
 
@@ -1294,7 +1301,8 @@ def sample_prior_predictive(samples=500, model=None, vars=None, random_seed=None
         Number of samples from the prior predictive to generate. Defaults to 500.
     model : Model (optional if in `with` context)
     vars : iterable
-        Variables for which to compute the posterior predictive samples.
+        A list of names of variables for which to compute the posterior predictive
+         samples.
         Defaults to `model.named_vars`.
     random_seed : int
         Seed for the random number generator.
@@ -1302,7 +1310,8 @@ def sample_prior_predictive(samples=500, model=None, vars=None, random_seed=None
     Returns
     -------
     dict
-        Dictionary with the variables as keys. The values are arrays of prior samples.
+        Dictionary with variable names as keys. The values are numpy arrays of prior
+         samples.
     """
     model = modelcontext(model)
 
