@@ -1,14 +1,14 @@
+from __future__ import division
+
 from collections import namedtuple
-import warnings
 
 import numpy as np
 import numpy.random as nr
-from scipy import stats, linalg
-import six
 
 from ..arraystep import Competence
-from pymc3.exceptions import SamplingError
-from .base_hmc import BaseHMC
+from .base_hmc import BaseHMC, HMCStepData, DivergenceInfo
+from .integration import IntegrationError
+from pymc3.backends.report import SamplerWarning, WarningType
 from pymc3.theanof import floatX
 from pymc3.vartypes import continuous_types
 
@@ -64,11 +64,12 @@ class NUTS(BaseHMC):
     - `step_size_bar`: The current best known step-size. After the tuning
       samples, the step size is set to this value. This should converge
       during tuning.
+    - `model_logp`: The model log-likelihood for this sample.
 
     References
     ----------
-    .. [1] Hoffman, Matthew D., & Gelman, Andrew. (2011). The No-U-Turn Sampler:
-       Adaptively Setting Path Lengths in Hamiltonian Monte Carlo.
+    .. [1] Hoffman, Matthew D., & Gelman, Andrew. (2011). The No-U-Turn
+       Sampler: Adaptively Setting Path Lengths in Hamiltonian Monte Carlo.
     """
 
     name = 'nuts'
@@ -86,12 +87,10 @@ class NUTS(BaseHMC):
         'energy_error': np.float64,
         'energy': np.float64,
         'max_energy_error': np.float64,
+        'model_logp': np.float64,
     }]
 
-    def __init__(self, vars=None, Emax=1000, target_accept=0.8,
-                 gamma=0.05, k=0.75, t0=10, adapt_step_size=True,
-                 max_treedepth=10, on_error='summary',
-                 early_max_treedepth=8,
+    def __init__(self, vars=None, max_treedepth=10, early_max_treedepth=8,
                  **kwargs):
         R"""Set up the No-U-Turn sampler.
 
@@ -101,19 +100,25 @@ class NUTS(BaseHMC):
         Emax : float, default 1000
             Maximum energy change allowed during leapfrog steps. Larger
             deviations will abort the integration.
-        target_accept : float (0,1), default .8
-            Try to find a step size such that the average acceptance
+        target_accept : float, default .8
+            Adapt the step size such that the average acceptance
             probability across the trajectories are close to target_accept.
             Higher values for target_accept lead to smaller step sizes.
+            Setting this to higher values like 0.9 or 0.99 can help
+            with sampling from difficult posteriors. Valid values are
+            between 0 and 1 (exclusive).
         step_scale : float, default 0.25
             Size of steps to take, automatically scaled down by `1/n**(1/4)`.
             If step size adaptation is switched off, the resulting step size
             is used. If adaptation is enabled, it is used as initial guess.
         gamma : float, default .05
-        k : float (.5,1) default .75
-            scaling of speed of adaptation
+        k : float, default .75
+            Parameter for dual averaging for step size adaptation. Values
+            between 0.5 and 1 (exclusive) are admissible. Higher values
+            correspond to slower adaptation.
         t0 : int, default 10
-            slows initial adaptation
+            Parameter for dual averaging. Higher values slow initial
+            adaptation.
         adapt_step_size : bool, default=True
             Whether step size adaptation should be enabled. If this is
             disabled, `k`, `t0`, `gamma` and `target_accept` are ignored.
@@ -132,12 +137,6 @@ class NUTS(BaseHMC):
             this will be interpreded as the mass or covariance matrix.
         is_cov : bool, default=False
             Treat the scaling as mass or covariance matrix.
-        on_error : {'summary', 'warn', 'raise'}, default='summary'
-            How to report problems during sampling.
-
-            * `summary`: Print one warning after sampling.
-            * `warn`: Print individual warnings as soon as they appear.
-            * `raise`: Raise an error on the first problem.
         potential : Potential, optional
             An object that represents the Hamiltonian with methods `velocity`,
             `energy`, and `random` methods. It can be specified instead
@@ -153,42 +152,13 @@ class NUTS(BaseHMC):
         `pm.sample` to the desired number of tuning steps.
         """
         super(NUTS, self).__init__(vars, **kwargs)
-        self.Emax = Emax
 
-        self.target_accept = target_accept
-        self.gamma = gamma
-        self.k = k
-        self.t0 = t0
-
-        self.h_bar = 0
-        self.mu = np.log(self.step_size * 10)
-        self.log_step_size = np.log(self.step_size)
-        self.log_step_size_bar = 0
-        self.m = 1
-        self.adapt_step_size = adapt_step_size
         self.max_treedepth = max_treedepth
         self.early_max_treedepth = early_max_treedepth
+        self._reached_max_treedepth = 0
 
-        self.tune = True
-        self.report = NutsReport(on_error, max_treedepth, target_accept)
-
-    def astep(self, q0):
-        """Perform a single NUTS iteration."""
-        p0 = self.potential.random()
-        start = self.integrator.compute_state(q0, p0)
-
-        if not np.isfinite(start.energy):
-            raise ValueError('Bad initial energy: %s. The model '
-                             'might be misspecified.' % start.energy)
-
-        if not self.adapt_step_size:
-            step_size = self.step_size
-        elif self.tune:
-            step_size = np.exp(self.log_step_size)
-        else:
-            step_size = np.exp(self.log_step_size_bar)
-
-        if self.tune and self.m < 200:
+    def _hamiltonian_step(self, start, p0, step_size):
+        if self.tune and self.iter_count < 200:
             max_treedepth = self.early_max_treedepth
         else:
             max_treedepth = self.max_treedepth
@@ -197,49 +167,41 @@ class NUTS(BaseHMC):
 
         for _ in range(max_treedepth):
             direction = logbern(np.log(0.5)) * 2 - 1
-            diverging_info, turning = tree.extend(direction)
-            q, q_grad = tree.proposal.q, tree.proposal.q_grad
+            divergence_info, turning = tree.extend(direction)
 
-            if diverging_info or turning:
-                if diverging_info:
-                    self.report._add_divergence(self.tune, *diverging_info)
+            if divergence_info or turning:
                 break
+        else:
+            if not self.tune:
+                self._reached_max_treedepth += 1
 
-        w = 1. / (self.m + self.t0)
-        self.h_bar = ((1 - w) * self.h_bar +
-                      w * (self.target_accept - tree.accept_sum * 1. / tree.n_proposals))
-
-        if self.tune:
-            self.log_step_size = self.mu - self.h_bar * np.sqrt(self.m) / self.gamma
-            mk = self.m ** -self.k
-            self.log_step_size_bar = mk * self.log_step_size + (1 - mk) * self.log_step_size_bar
-
-        self.m += 1
-
-        if self.tune:
-            self.potential.adapt(q, q_grad)
-
-        stats = {
-            'step_size': step_size,
-            'tune': self.tune,
-            'step_size_bar': np.exp(self.log_step_size_bar),
-            'diverging': bool(diverging_info),
-        }
-
-        stats.update(tree.stats())
-
-        return q, [stats]
+        stats = tree.stats()
+        accept_stat = stats['mean_tree_accept']
+        return HMCStepData(tree.proposal, accept_stat, divergence_info, stats)
 
     @staticmethod
-    def competence(var):
+    def competence(var, has_grad):
         """Check how appropriate this class is for sampling a random variable."""
-        if var.dtype in continuous_types:
+        if var.dtype in continuous_types and has_grad:
             return Competence.IDEAL
         return Competence.INCOMPATIBLE
 
+    def warnings(self):
+        warnings = super(NUTS, self).warnings()
+        n_samples = self._samples_after_tune
+        n_treedepth = self._reached_max_treedepth
+
+        if n_samples > 0 and n_treedepth / float(n_samples) > 0.05:
+            msg = ('The chain reached the maximum tree depth. Increase '
+                   'max_treedepth, increase target_accept or reparameterize.')
+            warn = SamplerWarning(WarningType.TREEDEPTH, msg, 'warn',
+                                  None, None, None)
+            warnings.append(warn)
+        return warnings
+
 
 # A proposal for the next position
-Proposal = namedtuple("Proposal", "q, q_grad, energy, p_accept")
+Proposal = namedtuple("Proposal", "q, q_grad, energy, p_accept, logp")
 
 # A subtree of the binary tree built by nuts.
 Subtree = namedtuple(
@@ -271,7 +233,8 @@ class _Tree(object):
         self.start_energy = np.array(start.energy)
 
         self.left = self.right = start
-        self.proposal = Proposal(start.q, start.q_grad, start.energy, 1.0)
+        self.proposal = Proposal(
+            start.q, start.q_grad, start.energy, 1.0, start.model_logp)
         self.depth = 0
         self.log_size = 0
         self.accept_sum = 0
@@ -285,7 +248,7 @@ class _Tree(object):
         If direction is larger than 0, extend it to the right, otherwise
         extend it to the left.
 
-        Return a tuple `(diverging, turning)` of type (bool, bool).
+        Return a tuple `(diverging, turning)` of type (DivergenceInfo, bool).
         `diverging` indicates, that the tree extension was aborted because
         the energy change exceeded `self.Emax`. `turning` indicates that
         the tree extension was stopped because the termination criterior
@@ -324,17 +287,9 @@ class _Tree(object):
         """Perform a leapfrog step and handle error cases."""
         try:
             right = self.integrator.step(epsilon, left)
-        except linalg.LinAlgError as err:
-            error_msg = "LinAlgError during leapfrog step."
+        except IntegrationError as err:
+            error_msg = str(err)
             error = err
-        except ValueError as err:
-            # Raised by many scipy.linalg functions
-            scipy_msg = "array must not contain infs or nans"
-            if len(err.args) > 0 and scipy_msg in err.args[0].lower():
-                error_msg = "Infs or nans in scipy.linalg during leapfrog step."
-                error = err
-            else:
-                raise
         else:
             energy_change = right.energy - self.start_energy
             if np.isnan(energy_change):
@@ -345,25 +300,30 @@ class _Tree(object):
             if np.abs(energy_change) < self.Emax:
                 p_accept = min(1, np.exp(-energy_change))
                 log_size = -energy_change
-                proposal = Proposal(right.q, right.q_grad, right.energy, p_accept)
-                tree = Subtree(right, right, right.p, proposal, log_size, p_accept, 1)
-                return tree, False, False
+                proposal = Proposal(
+                    right.q, right.q_grad, right.energy, p_accept, right.model_logp)
+                tree = Subtree(right, right, right.p,
+                               proposal, log_size, p_accept, 1)
+                return tree, None, False
             else:
-                error_msg = ("Energy change in leapfrog step is too large: %s. "
+                error_msg = ("Energy change in leapfrog step is too large: %s."
                              % energy_change)
                 error = None
         tree = Subtree(None, None, None, None, -np.inf, 0, 1)
-        return tree, (error_msg, error, left), False
+        divergance_info = DivergenceInfo(error_msg, error, left)
+        return tree, divergance_info, False
 
     def _build_subtree(self, left, depth, epsilon):
         if depth == 0:
             return self._single_step(left, epsilon)
 
-        tree1, diverging, turning = self._build_subtree(left, depth - 1, epsilon)
+        tree1, diverging, turning = self._build_subtree(
+            left, depth - 1, epsilon)
         if diverging or turning:
             return tree1, diverging, turning
 
-        tree2, diverging, turning = self._build_subtree(tree1.right, depth - 1, epsilon)
+        tree2, diverging, turning = self._build_subtree(
+            tree1.right, depth - 1, epsilon)
 
         left, right = tree1.left, tree2.right
 
@@ -384,7 +344,8 @@ class _Tree(object):
         accept_sum = tree1.accept_sum + tree2.accept_sum
         n_proposals = tree1.n_proposals + tree2.n_proposals
 
-        tree = Subtree(left, right, p_sum, proposal, log_size, accept_sum, n_proposals)
+        tree = Subtree(left, right, p_sum, proposal,
+                       log_size, accept_sum, n_proposals)
         return tree, diverging, turning
 
     def stats(self):
@@ -395,95 +356,5 @@ class _Tree(object):
             'energy': self.proposal.energy,
             'tree_size': self.n_proposals,
             'max_energy_error': self.max_energy_change,
+            'model_logp': self.proposal.logp,
         }
-
-
-class NutsReport(object):
-    def __init__(self, on_error, max_treedepth, target_accept):
-        if on_error not in ['summary', 'raise', 'warn']:
-            raise ValueError('Invalid value for on_error.')
-        self._on_error = on_error
-        self._max_treedepth = max_treedepth
-        self._target_accept = target_accept
-        self._chain_id = None
-        self._divs_tune = []
-        self._divs_after_tune = []
-
-    def _add_divergence(self, tuning, msg, error=None, point=None):
-        if tuning:
-            self._divs_tune.append((msg, error, point))
-        else:
-            self._divs_after_tune.append((msg, error, point))
-        if self._on_error == 'raise':
-            err = SamplingError('Divergence after tuning: %s Increase '
-                                'target_accept or reparameterize.' % msg)
-            six.raise_from(err, error)
-        elif self._on_error == 'warn':
-            warnings.warn('Divergence detected: %s Increase target_accept '
-                          'or reparameterize.' % msg)
-
-    def _check_len(self, tuning):
-        n = (~tuning).sum()
-        if n < 500:
-            warnings.warn('Chain %s contains only %s samples.'
-                          % (self._chain_id, n))
-        if np.all(tuning):
-            warnings.warn('Step size tuning was enabled throughout the whole '
-                          'trace. You might want to specify the number of '
-                          'tuning steps.')
-        if n > 0 and n == len(self._divs_after_tune):
-            warnings.warn('Chain %s contains only diverging samples. '
-                          'The model is probably misspecified.'
-                          % self._chain_id)
-
-    def _check_accept(self, accept):
-        mean_accept = np.mean(accept)
-        target_accept = self._target_accept
-        # Try to find a reasonable interval for acceptable acceptance
-        # probabilities. Finding this was mostry trial and error.
-        n_bound = min(100, len(accept))
-        n_good, n_bad = mean_accept * n_bound, (1 - mean_accept) * n_bound
-        lower, upper = stats.beta(n_good + 1, n_bad + 1).interval(0.95)
-        if target_accept < lower or target_accept > upper:
-            warnings.warn('The acceptance probability in chain %s does not '
-                          'match the target. It is %s, but should be close '
-                          'to %s. Try to increase the number of tuning steps.'
-                          % (self._chain_id, mean_accept, target_accept))
-
-    def _check_depth(self, depth):
-        if len(depth) == 0:
-            return
-        if np.mean(depth == self._max_treedepth) > 0.05:
-            warnings.warn('Chain %s reached the maximum tree depth. Increase '
-                          'max_treedepth, increase target_accept or '
-                          'reparameterize.' % self._chain_id)
-
-    def _check_divergence(self):
-        n_diverging = len(self._divs_after_tune)
-        if n_diverging > 0:
-            warnings.warn("Chain %s contains %s diverging samples after "
-                          "tuning. If increasing `target_accept` does not help "
-                          "try to reparameterize."
-                          % (self._chain_id, n_diverging))
-
-    def _finalize(self, strace):
-        """Print warnings for obviously problematic chains."""
-        self._chain_id = strace.chain
-
-        if strace.supports_sampler_stats:
-            tuning = strace.get_sampler_stats('tune')
-            if tuning.ndim == 2:
-                tuning = np.any(tuning, axis=-1)
-
-            accept = strace.get_sampler_stats('mean_tree_accept')
-            if accept.ndim == 2:
-                accept = np.mean(accept, axis=-1)
-
-            depth = strace.get_sampler_stats('depth')
-            if depth.ndim == 2:
-                depth = np.max(depth, axis=-1)
-
-            self._check_len(tuning)
-            self._check_depth(depth[~tuning])
-            self._check_accept(accept[~tuning])
-            self._check_divergence()
