@@ -14,7 +14,7 @@ def simple_model():
     with Model() as model:
         Normal('x', mu, tau=tau, shape=2, testval=tt.ones(2) * .1)
 
-    return model.test_point, model, (mu, tau ** -1)
+    return model.test_point, model, (mu, tau ** -.5)
 
 
 def simple_categorical():
@@ -34,7 +34,7 @@ def multidimensional_model():
     with Model() as model:
         Normal('x', mu, tau=tau, shape=(3, 2), testval=.1 * tt.ones((3, 2)))
 
-    return model.test_point, model, (mu, tau ** -1)
+    return model.test_point, model, (mu, tau ** -.5)
 
 
 def simple_arbitrary_det():
@@ -69,6 +69,16 @@ def simple_2model():
     return model.test_point, model
 
 
+def simple_2model_continuous():
+    mu = -2.1
+    tau = 1.3
+    with Model() as model:
+        x = pm.Normal('x', mu, tau=tau, testval=.1)
+        pm.Deterministic('logx', tt.log(x))
+        pm.Beta('y', alpha=1, beta=1, shape=2)
+    return model.test_point, model
+
+
 def mv_simple():
     mu = floatX_array([-.1, .5, 1.1])
     p = floatX_array([
@@ -147,3 +157,19 @@ def beta_bernoulli(n=2):
         pm.Beta('x', 3, 1, shape=n, transform=None)
         pm.Bernoulli('y', 0.5)
     return model.test_point, model, None
+
+
+def simple_normal(bounded_prior=False):
+    """Simple normal for testing MLE / MAP; probes issue #2482."""
+    x0 = 10.0
+    sd = 1.0
+    a, b = (9, 12)  # bounds for uniform RV, need non-symmetric to reproduce issue
+
+    with pm.Model() as model:
+        if bounded_prior:
+            mu_i = pm.Uniform("mu_i", a, b)
+        else:
+            mu_i = pm.Flat("mu_i")
+        pm.Normal("X_obs", mu=mu_i, sd=sd, observed=x0)
+
+    return model.test_point, model, None
